
/* Chrysalide - Outil d'analyse de fichiers binaires
 * ##FILE## - traduction d'instructions ARMv7
 *
 * Copyright (C) 2015 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


@title SUB (SP minus immediate)

@desc This instruction subtracts an immediate value from the SP value, and writes the result to the destination register.

@encoding (t1) {

	@half 1 0 1 1 0 0 0 0 1 imm7(7)

	@syntax <SP_0> <SP_1> <imm32>

	@conv {

		imm32 = ZeroExtend(imm7:'00', 32)
		SP_0 = Register(13)
		SP_1 = Register(13)

	}

}

@encoding (T2) {

	@word 1 1 1 1 0 i(1) 0 1 1 0 1 S(1) 1 1 0 1 0 imm3(3) Rd(4) imm8(8)

	@syntax <reg_D> <SP> <imm32>

	@conv {

		reg_D = Register(Rd)
		setflags = (S == '1')
		imm32 = ThumbExpandImm(i:imm3:imm8)
		SP = Register(13)

	}

	@rules {

		if (setflags); chk_call ExtendKeyword("s")
		chk_call ExtendKeyword(".w")

	}

}

@encoding (T3) {

	@word 1 1 1 1 0 i(1) 1 0 1 0 1 0 1 1 0 1 0 imm3(3) Rd(4) imm8(8)

	@syntax "subw" <reg_D> <SP> <imm32>

	@conv {

		reg_D = Register(Rd)
		imm32 = ZeroExtend(i:imm3:imm8, 32)
		SP = Register(13)

	}

}

@encoding (A1) {

	@word cond(4) 0 0 1 0 0 1 0 S(1) 1 1 0 1 Rd(4) imm12(12)

	@syntax <reg_D> <SP> <imm32>

	@conv {

		reg_D = Register(Rd)
		setflags = (S == '1')
		imm32 = ARMExpandImm(imm12)
		SP = Register(13)

	}

	@rules {

		if (setflags); chk_call ExtendKeyword("s")
		chk_call StoreCondition(cond)

	}

}

